import pandas as pd
import matplotlib.pyplot as plt
import matplotlib.dates as mdates

# read the downloaded csv file
df = pd.read_csv("opendata-kfue-odl_7t.csv", sep=';', decimal=',', on_bad_lines='warn', encoding="ISO-8859-1")

# Get the rows for the abbreviation to name matching
df_colnames = df.take(range(171,255))
# drop all columns except the first
df_colnames.drop(columns=df.columns[1:], inplace=True)

# Create a dictionary to translate from abbreviation to the actual name
col_dict = {}
for row in df_colnames.iloc[:,0]:
    place_abbreviation = row[:12]
    # We are not interested in Weather data so we dont need them in the dictionary
    if "MET" not in place_abbreviation and "TIS" not in place_abbreviation:
        place = str(row[15:65]).strip().removeprefix("KF"+ chr(ord("Ü"))+"-Sonde")
        # print(place_abbreviation, " = ", place)
        col_dict[place_abbreviation] = place

# Extract the data rows and drop the last column because it is an empty column
df_data = df.take(range(1, 169))
df_data.drop(columns=df.columns[-1], inplace=True)

# Remove the unused columns (everything except Radiation)
for col in df_data.columns:
    if "MET" in col or "TIS" in col:
        df_data.drop(columns=col, inplace=True)
    elif col != "Datum/Uhrzeit":
        # The values are in scientific notation (7,095e-02) and need to be converted
        # Convert all values to strings where the ',' is replaced by the decimal indicator '.'
        # Further use pandas library to convert them to numeric values
        # the method apply uses a given function on every element of the specified column of the dataframe
        df_data[col]= df_data[col].apply(lambda x: str(x).replace(",", ".")).apply(pd.to_numeric, errors='coerce')

# Drop all columns where all values consist of NaN
df_data.dropna(axis=1, how='all', inplace=True)

# Create a Boxplot
df_data.drop(columns=df.columns[0], inplace=True)
plt.figure(figsize=(18,18))
plt.title("Power plant monitoring")
plt.boxplot(df_data.iloc[:,1:], tick_labels=[col_dict[x] for x in df_data.columns[1:]])
plt.xticks(rotation=90)
# \u00B5 is equal to the micro character
plt.ylabel("\u00B5S/h")
plt.grid()
plt.savefig("boxplot.png")
